window.Gleam.initSearch([{"doc":"omnimessage_server","title":"omnimessage_server","content":"# omnimessage_server\n\n**Seamless communication with Lustre applications**\n\n[![Package Version](https://img.shields.io/hexpm/v/omnimessage_server)](https://hex.pm/packages/omnimessage_server)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/omnimessage_server/)\n\nA main challenge of maintaining a client side application is state\nsynchronization. Usually, a complete separation is ideal, where client state is\nmanaged exclusively at the client, and server state exclusively at the server.\nThere&#39;s nuance to this take, but it&#39;s a good general rule to follow.\n\n[Lustre](https://hexdocs.pm/lustre/) exceles at this with having both the power\nof a full-blown single page application, and the flexibility of using server\ncomponents for a LiveView/HTMX like solution to state fully owned by the server.\n\nThis approach becomes brittle when the state is shared between both the client\nand server. In a chat app, for example, the messages are state owned by the\nserver, but you still need to display a message as `Sending...` before the\nserver knows it exists. This is usually where LiveView or HTMLX will recommend\nyou sprinkle some Javascript. Lustre has an easier time dealing with this, but\nit can become quite cumbersome to manage the different HTTP requests and\nwebsocket connections.\n\nThis is where Omnimessage comes in. When paired with a properly set up client\n(see [omnimessage_luster](https://hexdocs.pm/omnimessage_lustre)), all you need\nto do to communicate is handle Lustre messages.\n\nYou can do that via websockets:\n\n```gleam\nimport omnimessage_server as omniserver\n\n  case request.path_segments(req), req.method {\n    [&quot;omni-ws&quot;], http.Get -&gt;\n      omniserver.mist_websocket_pipe(\n        req,\n        // this is for encoding/decoding, supplied by you\n        encoder_decoder(),\n        // message handler, recieves a message from the client and whatever\n        // it returns will be sent back to the client\n        handler,\n        // error handler\n        fn(_) { Nil },\n      )\n  }\n```\nvia HTTP:\n\n```gleam\nimport omnimessage_server as omniserver\n\nfn wisp_handler(req, ctx) {\n  use &lt;- cors_middleware(req)\n  use &lt;- static_middleware(req)\n\n  // For handling HTTP transports\n  use &lt;- omniserver.wisp_http_middleware(\n    req,\n    &quot;/omni-http&quot;,\n    encoder_decoder(),\n    handler,\n  )\n\n  // ...rest of handler\n}\n```\n\nor via full-blown lustre server component, communicating via websockets:\n\n```gleam\nimport omnimessage_server as omniserver\n\n  case request.path_segments(req), req.method {\n    [&quot;omni-ws&quot;], http.Get -&gt;\n      // chat.app() is a special lustre server component that&#39;ll recieve\n      // message form the client and can dispatch message to answer\n      omniserver.mist_websocket_application(req, chat.app(), ctx, fn(_) { Nil })\n  }\n```\n\n\nFurther documentation can be found at &lt;https://hexdocs.pm/omnimessage_server&gt;.\n","url":"index.html"},{"doc":"omnimessage/server","title":"App","content":"pub opaque type App(flags, model, msg)\n\n\nSynonyms:\nomnimessage/server.App\nomnimessage/server App","url":"omnimessage/server.html#App"},{"doc":"omnimessage/server","title":"ComposedApp","content":"pub opaque type ComposedApp(\n  flags,\n  model,\n  msg,\n  encoding,\n  decode_error,\n)\n\n\nSynonyms:\nomnimessage/server.ComposedApp\nomnimessage/server ComposedApp","url":"omnimessage/server.html#ComposedApp"},{"doc":"omnimessage/server","title":"EncoderDecoder","content":"pub type EncoderDecoder(msg, encoding, decode_error) {\n  EncoderDecoder(\n    encode: fn(msg) -&gt; Result(encoding, Nil),\n    decode: fn(encoding) -&gt; Result(msg, decode_error),\n  )\n}\n Holds decode and encode functions for omnimessage messages. Decode errors\n will be called back for you to handle, while Encode errors are interpreted\n as &quot;skip this message&quot; -- no error will be raised for them and they won&#39;t\n be sent over.\n\n Since an `EncoderDecoder` is expected to receive the whole message type of\n an application, but usually will ignore messages that aren&#39;t shared, it&#39;s\n best to define it as a thin wrapper around shared encoders/decoders:\n\n \n // Holds shared message types, encoders and decoders\n import shared\n\n let encoder_decoder =\n   EncoderDecoder(\n     fn(msg) {\n       case msg {\n         // Messages must be encodable\n         ClientMessage(message) -&gt; Ok(shared.encode_client_message(message))\n         // Return Error(Nil) for messages you don&#39;t want to send out\n         _ -&gt; Error(Nil)\n       }\n     },\n     fn(encoded_msg) {\n       // Unsupported messages will cause TransportError(DecodeError(error))\n       shared.decode_server_message(encoded_msg)\n       |&gt; result.map(ServerMessage)\n     },\n   )\n \n\nEncoderDecoder(\n  encode: fn(msg) -&gt; Result(encoding, Nil),\n  decode: fn(encoding) -&gt; Result(msg, decode_error),\n)\n\n\nSynonyms:\nomnimessage/server.EncoderDecoder\nomnimessage/server EncoderDecoder","url":"omnimessage/server.html#EncoderDecoder"},{"doc":"omnimessage/server","title":"application","content":"pub fn application(\n  init: fn(a) -&gt; #(b, Effect(c)),\n  update: fn(b, c) -&gt; #(b, Effect(c)),\n  encoder_decoder: EncoderDecoder(c, d, e),\n) -&gt; ComposedApp(a, b, c, d, e)\n This creates a version of a Lustre application that can be used in\n `omnimessage/server.start_actor` (see below). A view is not necessary, as\n this application will never render anything.\n\nSynonyms:\nomnimessage/server.application\nomnimessage/server application","url":"omnimessage/server.html#application"},{"doc":"omnimessage/server","title":"dispatch","content":"pub fn dispatch(message: a) -&gt; Action(a, b)\n Dispatch a message to a running application&#39;s `update` function.\n\nSynonyms:\nomnimessage/server.dispatch\nomnimessage/server dispatch","url":"omnimessage/server.html#dispatch"},{"doc":"omnimessage/server","title":"mist_websocket_application","content":"pub fn mist_websocket_application(\n  req: Request(Connection),\n  app: ComposedApp(a, b, c, String, d),\n  flags: a,\n  on_error: fn(d) -&gt; Nil,\n) -&gt; Response(ResponseData)\n A mist websocket handler to automatically responsd to omnimessage messages\n via a Lustre server component. The server component can then be used\n similarly to one created by an `omnimessage/lustre` and handle the messages\n via update, dispatch, and effects.\n\n Return this as a response to the websocket init request.\n\n   - `req`       The mist request\n   - `app`       An application created with `omnimessage/server.application`\n   - `flags`     Flags to hand to the application&#39;s `init`\n   - `on_error`  For handling decode errors\n\n See a full example using this in the Readme or in the examples folder.\n\nSynonyms:\nomnimessage/server.mist_websocket_application\nomnimessage/server mist_websocket_application","url":"omnimessage/server.html#mist_websocket_application"},{"doc":"omnimessage/server","title":"mist_websocket_pipe","content":"pub fn mist_websocket_pipe(\n  req: Request(Connection),\n  encoder_decoder: EncoderDecoder(a, String, b),\n  handler: fn(a) -&gt; a,\n  on_error: fn(b) -&gt; Nil,\n) -&gt; Response(ResponseData)\n A mist websocket handler to automatically responsd to omnimessage messages.\n\n Return this as a response to the websocket init request.\n\n   - `req`              The mist request\n   - `encoder_decoder`  For encoding and decoding messages\n   - `handler`          For handling the incoming messages\n   - `on_error`         For handling decode errors\n\n See a full example using this in the Readme or in the examples folder.\n\nSynonyms:\nomnimessage/server.mist_websocket_pipe\nomnimessage/server mist_websocket_pipe","url":"omnimessage/server.html#mist_websocket_pipe"},{"doc":"omnimessage/server","title":"pipe","content":"pub fn pipe(\n  msg: a,\n  encoder_decoder: EncoderDecoder(b, a, c),\n  handler: fn(b) -&gt; b,\n) -&gt; Result(Option(a), c)\n A utility function for easily handling messages:\n\n \n let out_msg = pipe(in_msg, encoder_decoder, handler)\n \n\nSynonyms:\nomnimessage/server.pipe\nomnimessage/server pipe","url":"omnimessage/server.html#pipe"},{"doc":"omnimessage/server","title":"shutdown","content":"pub fn shutdown() -&gt; Action(a, b)\n Instruct a running application to shut down.\n\nSynonyms:\nomnimessage/server.shutdown\nomnimessage/server shutdown","url":"omnimessage/server.html#shutdown"},{"doc":"omnimessage/server","title":"start_actor","content":"pub fn start_actor(\n  app: ComposedApp(a, b, c, d, e),\n  with flags: a,\n) -&gt; Result(Subject(Action(c, ServerComponent)), Error)\n This is a beefed up version of `lustre.start_actor` that allows subscribing\n to messages dispatched inside the runtime.\n\n This is what enables using a Lustre server component for communication,\n powering `mist_websocket_application()` below.\n\nSynonyms:\nomnimessage/server.start_actor\nomnimessage/server start_actor","url":"omnimessage/server.html#start_actor"},{"doc":"omnimessage/server","title":"subscribe","content":"pub fn subscribe(\n  id: String,\n  dispatch: fn(a) -&gt; Nil,\n) -&gt; Action(a, b)\n This action subscribes to updates in a running application.\n\nSynonyms:\nomnimessage/server.subscribe\nomnimessage/server subscribe","url":"omnimessage/server.html#subscribe"},{"doc":"omnimessage/server","title":"wisp_http_middleware","content":"pub fn wisp_http_middleware(\n  req: Request(Connection),\n  path: String,\n  encoder_decoder: EncoderDecoder(a, String, b),\n  handler: fn(a) -&gt; a,\n  fun: fn() -&gt; Response(Body),\n) -&gt; Response(Body)\n A wisp middleware to automatically handle HTTP POST omnimessage messages.\n\n   - `req`              The wisp request\n   - `path`             The path to which messages are POSTed\n   - `encoder_decoder`  For encoding and decoding messages\n   - `handler`          For handling the incoming messages\n\n See a full example using this in the Readme or in the examples folder.\n\nSynonyms:\nomnimessage/server.wisp_http_middleware\nomnimessage/server wisp_http_middleware","url":"omnimessage/server.html#wisp_http_middleware"},{"doc":"omnimessage/server","title":"omnimessage/server","content":"","url":"omnimessage/server.html"}]);